@@ -329,23 +329,16 @@ void EditProfileDialog::setupGeneralPage(const Profile::Ptr profile)
 {
     // basic profile options
     {
-        _ui->emptyNameWarningWidget->setWordWrap(false);
-        _ui->emptyNameWarningWidget->setCloseButtonVisible(false);
-        _ui->emptyNameWarningWidget->setMessageType(KMessageWidget::Warning);
+        _ui->profileNameEdit->setPlaceholderText(i18nc("@label:textbox", "Enter descriptive label"));
 
         ProfileGroup::Ptr group = profile->asGroup();
         if (!group || group->profiles().count() < 2) {
             _ui->profileNameEdit->setText(profile->name());
             _ui->profileNameEdit->setClearButtonEnabled(true);
-
-            _ui->emptyNameWarningWidget->setVisible(profile->name().isEmpty());
-            _ui->emptyNameWarningWidget->setText(i18n("Profile name is empty."));
         } else {
             _ui->profileNameEdit->setText(groupProfileNames(group, -1));
             _ui->profileNameEdit->setEnabled(false);
             _ui->profileNameLabel->setEnabled(false);
-
-            _ui->emptyNameWarningWidget->setVisible(false);
         }
     }
 
@@ -491,8 +484,6 @@ void EditProfileDialog::selectIcon()
 
 void EditProfileDialog::profileNameChanged(const QString &name)
 {
-    _ui->emptyNameWarningWidget->setVisible(name.isEmpty());
-
     updateTempProfileProperty(Profile::Name, name);
     updateTempProfileProperty(Profile::UntranslatedName, name);
     updateCaption(_tempProfile);
@@ -653,6 +644,11 @@ void EditProfileDialog::toggleMouseWheelZoom(bool enable)
     updateTempProfileProperty(Profile::MouseWheelZoomEnabled, enable);
 }
 
+void EditProfileDialog::toggleAlternateScrolling(bool enable)
+{
+    updateTempProfileProperty(Profile::AlternateScrolling, enable);
+}
+
 void EditProfileDialog::updateColorSchemeList(const QString &selectedColorSchemeName)
 {
     if (_ui->colorSchemeList->model() == nullptr) {
@@ -698,17 +694,12 @@ void EditProfileDialog::updateColorSchemeList(const QString &selectedColorScheme
     }
 }
 
-void EditProfileDialog::updateKeyBindingsList(bool selectCurrentTranslator)
+void EditProfileDialog::updateKeyBindingsList(const QString &selectKeyBindingsName)
 {
     if (_ui->keyBindingList->model() == nullptr) {
         _ui->keyBindingList->setModel(new QStandardItemModel(this));
     }
 
-    const QString &name = lookupProfile()->keyBindings();
-
-    KeyboardTranslatorManager *keyManager = KeyboardTranslatorManager::instance();
-    const KeyboardTranslator *currentTranslator = keyManager->findTranslator(name);
-
     QStandardItemModel *model = qobject_cast<QStandardItemModel *>(_ui->keyBindingList->model());
 
     Q_ASSERT(model);
@@ -717,9 +708,9 @@ void EditProfileDialog::updateKeyBindingsList(bool selectCurrentTranslator)
 
     QStandardItem *selectedItem = nullptr;
 
-    QStringList translatorNames = keyManager->allTranslators();
-    foreach (const QString &translatorName, translatorNames) {
-        const KeyboardTranslator *translator = keyManager->findTranslator(translatorName);
+    const QStringList &translatorNames = _keyManager->allTranslators();
+    for (const QString &translatorName : translatorNames) {
+        const KeyboardTranslator *translator = _keyManager->findTranslator(translatorName);
         if (translator == nullptr) {
             continue;
         }
@@ -727,10 +718,11 @@ void EditProfileDialog::updateKeyBindingsList(bool selectCurrentTranslator)
         QStandardItem *item = new QStandardItem(translator->description());
         item->setEditable(false);
         item->setData(QVariant::fromValue(translator), Qt::UserRole + 1);
+        item->setData(QVariant::fromValue(_keyManager->findTranslatorPath(translatorName)), Qt::ToolTipRole);
         item->setData(QVariant::fromValue(_profile->font()), Qt::UserRole + 2);
         item->setIcon(QIcon::fromTheme(QStringLiteral("preferences-desktop-keyboard")));
 
-        if (translator == currentTranslator) {
+        if (selectKeyBindingsName == translatorName) {
             selectedItem = item;
         }
 
@@ -739,7 +731,7 @@ void EditProfileDialog::updateKeyBindingsList(bool selectCurrentTranslator)
 
     model->sort(0);
 
-    if (selectCurrentTranslator && (selectedItem != nullptr)) {
+    if (selectedItem != nullptr) {
         _ui->keyBindingList->selectionModel()->setCurrentIndex(selectedItem->index(),
                                                                QItemSelectionModel::Select);
     }
@@ -1068,8 +1060,21 @@ void EditProfileDialog::updateColorSchemeButtons()
 
 void EditProfileDialog::updateKeyBindingsButtons()
 {
-    enableIfNonEmptySelection(_ui->editKeyBindingsButton, _ui->keyBindingList->selectionModel());
-    enableIfNonEmptySelection(_ui->removeKeyBindingsButton, _ui->keyBindingList->selectionModel());
+    QModelIndexList selected = _ui->keyBindingList->selectionModel()->selectedIndexes();
+
+    if (!selected.isEmpty()) {
+        _ui->editKeyBindingsButton->setEnabled(true);
+
+        const QString &name = selected.first().data(Qt::UserRole + 1).value<const KeyboardTranslator *>()->name();
+
+        bool isResettable = _keyManager->isTranslatorResettable(name);
+        _ui->resetKeyBindingsButton->setEnabled(isResettable);
+
+        bool isDeletable = _keyManager->isTranslatorDeletable(name);
+
+        // if a key bindings scheme can be reset then it can't be deleted
+        _ui->removeKeyBindingsButton->setEnabled(isDeletable && !isResettable);
+    }
 }
 
 void EditProfileDialog::enableIfNonEmptySelection(QWidget *widget, QItemSelectionModel *selectionModel)
@@ -1129,8 +1134,12 @@ void EditProfileDialog::updateButtonApply()
                 userModified = true;
                 break;
             }
-            // for not-previewed property
-        } else if ((value != _profile->property<QVariant>(property))) {
+        // for not-previewed property
+        //
+        // for the Profile::KeyBindings property, if it's set in the _tempProfile
+        // then the user opened the edit key bindings dialog and clicked
+        // OK, and could have add/removed a key bindings rule
+        } else if (property == Profile::KeyBindings || (value != _profile->property<QVariant>(property))) {
             userModified = true;
             break;
         }
@@ -1142,7 +1151,7 @@ void EditProfileDialog::updateButtonApply()
 void EditProfileDialog::setupKeyboardPage(const Profile::Ptr /* profile */)
 {
     // setup translator list
-    updateKeyBindingsList(true);
+    updateKeyBindingsList(lookupProfile()->keyBindings());
 
     connect(_ui->keyBindingList->selectionModel(),
             &QItemSelectionModel::selectionChanged, this,
@@ -1150,12 +1159,18 @@ void EditProfileDialog::setupKeyboardPage(const Profile::Ptr /* profile */)
     connect(_ui->newKeyBindingsButton, &QPushButton::clicked, this,
             &Konsole::EditProfileDialog::newKeyBinding);
 
+    _ui->editKeyBindingsButton->setEnabled(false);
+    _ui->removeKeyBindingsButton->setEnabled(false);
+    _ui->resetKeyBindingsButton->setEnabled(false);
+
     updateKeyBindingsButtons();
 
     connect(_ui->editKeyBindingsButton, &QPushButton::clicked, this,
             &Konsole::EditProfileDialog::editKeyBinding);
     connect(_ui->removeKeyBindingsButton, &QPushButton::clicked, this,
             &Konsole::EditProfileDialog::removeKeyBinding);
+    connect(_ui->resetKeyBindingsButton, &QPushButton::clicked, this,
+            &Konsole::EditProfileDialog::resetKeyBindings);
 }
 
 void EditProfileDialog::keyBindingSelected()
@@ -1195,60 +1210,24 @@ void EditProfileDialog::showKeyBindingEditor(bool isNewTranslator)
     if (!selected.isEmpty()) {
         translator = model->data(selected.first(), Qt::UserRole + 1).value<const KeyboardTranslator *>();
     } else {
-        translator = KeyboardTranslatorManager::instance()->defaultTranslator();
+        translator = _keyManager->defaultTranslator();
     }
 
     Q_ASSERT(translator);
 
-    QPointer<QDialog> dialog = new QDialog(this);
-    auto buttonBox = new QDialogButtonBox(dialog);
-    buttonBox->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
-    connect(buttonBox, &QDialogButtonBox::accepted, dialog.data(), &QDialog::accept);
-    connect(buttonBox, &QDialogButtonBox::rejected, dialog.data(), &QDialog::reject);
-
-    if (isNewTranslator) {
-        dialog->setWindowTitle(i18n("New Key Binding List"));
-    } else {
-        dialog->setWindowTitle(i18n("Edit Key Binding List"));
-    }
-
-    auto editor = new KeyBindingEditor;
+    auto editor = new KeyBindingEditor(this);
 
     if (translator != nullptr) {
-        editor->setup(translator);
+        editor->setup(translator, lookupProfile()->keyBindings(), isNewTranslator);
     }
 
-    if (isNewTranslator) {
-        editor->setDescription(i18n("New Key Binding List"));
-    }
+    connect(editor, &Konsole::KeyBindingEditor::updateKeyBindingsListRequest,
+            this, &Konsole::EditProfileDialog::updateKeyBindingsList);
 
-    auto layout = new QVBoxLayout;
-    layout->addWidget(editor);
-    layout->addWidget(buttonBox);
-    dialog->setLayout(layout);
-    // see also the size set in the KeyBindingEditor constructor
-    dialog->setMinimumSize(480, 430);
-    dialog->resize(500, 500);
+    connect(editor, &Konsole::KeyBindingEditor::updateTempProfileKeyBindingsRequest,
+            this, &Konsole::EditProfileDialog::updateTempProfileProperty);
 
-    if (dialog->exec() == QDialog::Accepted) {
-        auto newTranslator = new KeyboardTranslator(*editor->translator());
-
-        if (isNewTranslator) {
-            newTranslator->setName(newTranslator->description());
-        }
-
-        KeyboardTranslatorManager::instance()->addTranslator(newTranslator);
-
-        updateKeyBindingsList();
-
-        const QString &currentTranslator = lookupProfile()
-                                           ->property<QString>(Profile::KeyBindings);
-
-        if (newTranslator->name() == currentTranslator) {
-            updateTempProfileProperty(Profile::KeyBindings, newTranslator->name());
-        }
-    }
-    delete dialog;
+    editor->exec();
 }
 
 void EditProfileDialog::newKeyBinding()
@@ -1261,28 +1240,34 @@ void EditProfileDialog::editKeyBinding()
     showKeyBindingEditor(false);
 }
 
-void EditProfileDialog::setupCheckBoxes(BooleanOption *options, const Profile::Ptr profile)
+void EditProfileDialog::resetKeyBindings()
 {
-    while (options->button != nullptr) {
-        options->button->setChecked(profile->property<bool>(options->property));
-        connect(options->button, SIGNAL(toggled(bool)), this, options->slot);
+    QModelIndexList selected = _ui->keyBindingList->selectionModel()->selectedIndexes();
 
-        ++options;
+    if (!selected.isEmpty()) {
+        const QString &name = selected.first().data(Qt::UserRole + 1).value<const KeyboardTranslator *>()->name();
+
+        _keyManager->deleteTranslator(name);
+        // find and load the translator
+        _keyManager->findTranslator(name);
+
+        updateKeyBindingsList(name);
     }
 }
 
-void EditProfileDialog::setupRadio(RadioOption *possibilities, int actual)
+void EditProfileDialog::setupCheckBoxes(const QVector<BooleanOption>& options, const Profile::Ptr profile)
 {
-    while (possibilities->button != nullptr) {
-        if (possibilities->value == actual) {
-            possibilities->button->setChecked(true);
-        } else {
-            possibilities->button->setChecked(false);
-        }
+    for(const auto& option : options) {
+        option.button->setChecked(profile->property<bool>(option.property));
+        connect(option.button, SIGNAL(toggled(bool)), this, option.slot);
+    }
+}
 
-        connect(possibilities->button, SIGNAL(clicked()), this, possibilities->slot);
-
-        ++possibilities;
+void EditProfileDialog::setupRadio(const QVector<RadioOption>& possibilities, int actual)
+{
+    for(const auto& possibility : possibilities) {
+        possibility.button->setChecked(possibility.value == actual);
+        connect(possibility.button, SIGNAL(clicked()), this, possibility.slot);
     }
 }
 
@@ -1291,10 +1276,9 @@ void EditProfileDialog::setupScrollingPage(const Profile::Ptr profile)
     // setup scrollbar radio
     int scrollBarPosition = profile->property<int>(Profile::ScrollBarPosition);
 
-    RadioOption positions[] = { {_ui->scrollBarHiddenButton, Enum::ScrollBarHidden, SLOT(hideScrollBar())},
+    const auto positions = QVector<RadioOption>{ {_ui->scrollBarHiddenButton, Enum::ScrollBarHidden, SLOT(hideScrollBar())},
                                 {_ui->scrollBarLeftButton, Enum::ScrollBarLeft, SLOT(showScrollBarLeft())},
-                                {_ui->scrollBarRightButton, Enum::ScrollBarRight, SLOT(showScrollBarRight())},
-                                {nullptr, 0, nullptr}};
+                                {_ui->scrollBarRightButton, Enum::ScrollBarRight, SLOT(showScrollBarRight())}};
 
     setupRadio(positions, scrollBarPosition);
 
@@ -1311,10 +1295,9 @@ void EditProfileDialog::setupScrollingPage(const Profile::Ptr profile)
     // setup scrollpageamount type radio
     int scrollFullPage = profile->property<int>(Profile::ScrollFullPage);
 
-    RadioOption pageamounts[] = {
+    const auto pageamounts = QVector<RadioOption>{
         {_ui->scrollHalfPage, Enum::ScrollPageHalf, SLOT(scrollHalfPage())},
-        {_ui->scrollFullPage, Enum::ScrollPageFull, SLOT(scrollFullPage())},
-        {nullptr, 0, nullptr}
+        {_ui->scrollFullPage, Enum::ScrollPageFull, SLOT(scrollFullPage())}
     };
 
     setupRadio(pageamounts, scrollFullPage);
@@ -1361,7 +1344,7 @@ void EditProfileDialog::scrollHalfPage()
 
 void EditProfileDialog::setupMousePage(const Profile::Ptr profile)
 {
-    BooleanOption options[] = {
+    const auto options = QVector<BooleanOption>{
         {
             _ui->underlineLinksButton, Profile::UnderlineLinksEnabled,
             SLOT(toggleUnderlineLinks(bool))
@@ -1398,17 +1381,18 @@ void EditProfileDialog::setupMousePage(const Profile::Ptr profile)
             _ui->dropUrlsAsText, Profile::DropUrlsAsText,
             SLOT(toggleDropUrlsAsText(bool))
         },
-        { nullptr, Profile::Property(0), nullptr }
+        {
+            _ui->enableAlternateScrollingButton, Profile::AlternateScrolling,
+            SLOT(toggleAlternateScrolling(bool))
+        }
     };
     setupCheckBoxes(options, profile);
 
     // setup middle click paste mode
     const int middleClickPasteMode = profile->property<int>(Profile::MiddleClickPasteMode);
-    RadioOption pasteModes[] = {
+    const auto pasteModes = QVector<RadioOption> {
         {_ui->pasteFromX11SelectionButton, Enum::PasteFromX11Selection, SLOT(pasteFromX11Selection())},
-        {_ui->pasteFromClipboardButton, Enum::PasteFromClipboard, SLOT(pasteFromClipboard())},
-        {nullptr, 0, nullptr}
-    };
+        {_ui->pasteFromClipboardButton, Enum::PasteFromClipboard, SLOT(pasteFromClipboard())}    };
     setupRadio(pasteModes, middleClickPasteMode);
 
     // interaction options
@@ -1429,7 +1413,7 @@ void EditProfileDialog::setupMousePage(const Profile::Ptr profile)
 
 void EditProfileDialog::setupAdvancedPage(const Profile::Ptr profile)
 {
-    BooleanOption options[] = {
+    const auto options = QVector<BooleanOption>{
         {
             _ui->enableBlinkingTextButton, Profile::BlinkingTextEnabled,
             SLOT(toggleBlinkingText(bool))
@@ -1445,8 +1429,7 @@ void EditProfileDialog::setupAdvancedPage(const Profile::Ptr profile)
         {
             _ui->enableBidiRenderingButton, Profile::BidiRenderingEnabled,
             SLOT(togglebidiRendering(bool))
-        },
-        { nullptr, Profile::Property(0), nullptr }
+        }
     };
     setupCheckBoxes(options, profile);
 
