@@ -63,7 +63,10 @@ using namespace Konsole;
 
 MainWindow::MainWindow() :
     KXmlGuiWindow(),
+    _viewManager(nullptr),
     _bookmarkHandler(nullptr),
+    _toggleMenuBarAction(nullptr),
+    _newTabMenuAction(nullptr),
     _pluggedController(nullptr),
     _menuBarInitialVisibility(true),
     _menuBarInitialVisibilityApplied(false)
@@ -224,8 +227,6 @@ void MainWindow::disconnectController(SessionController *controller)
     if (controller->isValid()) {
         guiFactory()->removeClient(controller);
     }
-
-    controller->setSearchBar(nullptr);
 }
 
 void MainWindow::activeViewChanged(SessionController *controller)
@@ -236,7 +237,7 @@ void MainWindow::activeViewChanged(SessionController *controller)
     connect(bookmarkHandler(), &Konsole::BookmarkHandler::openUrl, controller,
             &Konsole::SessionController::openUrl);
 
-    if (_pluggedController != nullptr) {
+    if (!_pluggedController.isNull()) {
         disconnectController(_pluggedController);
     }
 
@@ -256,9 +257,6 @@ void MainWindow::activeViewChanged(SessionController *controller)
     controller->setShowMenuAction(_toggleMenuBarAction);
     guiFactory()->addClient(controller);
 
-    // set the current session's search bar
-    controller->setSearchBar(searchBar());
-
     // update session title to match newly activated session
     activeViewTitleChanged(controller);
 
@@ -274,7 +272,7 @@ void MainWindow::activeViewTitleChanged(ViewProperties *properties)
 
 void MainWindow::updateWindowCaption()
 {
-    if (_pluggedController == nullptr) {
+    if (_pluggedController.isNull()) {
         return;
     }
 
@@ -284,9 +282,12 @@ void MainWindow::updateWindowCaption()
     // use tab title as caption by default
     QString caption = title;
 
-    // use window title as caption only when enabled and it is not empty
-    if (KonsoleSettings::showWindowTitleOnTitleBar() && !userTitle.isEmpty()) {
-        caption = userTitle;
+    // use window title as caption when this setting is enabled
+    // if the userTitle is empty, use a blank space (using an empty string
+    // removes the dash — before the application name; leaving the dash
+    // looks better)
+    if (KonsoleSettings::showWindowTitleOnTitleBar()) {
+        !userTitle.isEmpty() ? caption = userTitle : caption = QStringLiteral(" ");
     }
 
     if (KonsoleSettings::showAppNameOnTitleBar()) {
@@ -298,16 +299,11 @@ void MainWindow::updateWindowCaption()
 
 void MainWindow::updateWindowIcon()
 {
-    if ((_pluggedController != nullptr) && !_pluggedController->icon().isNull()) {
+    if ((!_pluggedController.isNull()) && !_pluggedController->icon().isNull()) {
         setWindowIcon(_pluggedController->icon());
     }
 }
 
-IncrementalSearchBar *MainWindow::searchBar() const
-{
-    return _viewManager->searchBar();
-}
-
 void MainWindow::setupActions()
 {
     KActionCollection *collection = actionCollection();
@@ -443,7 +439,7 @@ void MainWindow::profileListChanged(const QList<QAction *> &sessionActions)
 
 QString MainWindow::activeSessionDir() const
 {
-    if (_pluggedController != nullptr) {
+    if (!_pluggedController.isNull()) {
         if (Session *session = _pluggedController->session()) {
             // For new tabs to get the correct working directory,
             // force the updating of the currentWorkingDirectory.
@@ -507,7 +503,7 @@ Session *MainWindow::createSession(Profile::Ptr profile, const QString &director
     // doesn't suffer a change in terminal size right after the session
     // starts.  Some applications such as GNU Screen and Midnight Commander
     // don't like this happening
-    createView(session);
+    _viewManager->createView(session);
 
     return session;
 }
@@ -537,16 +533,11 @@ Session *MainWindow::createSSHSession(Profile::Ptr profile, const QUrl &url)
     // doesn't suffer a change in terminal size right after the session
     // starts.  some applications such as GNU Screen and Midnight Commander
     // don't like this happening
-    createView(session);
+    _viewManager->createView(session);
 
     return session;
 }
 
-void MainWindow::createView(Session *session)
-{
-    _viewManager->createView(session);
-}
-
 void MainWindow::setFocus()
 {
     _viewManager->activeView()->setFocus();
@@ -569,7 +560,9 @@ bool MainWindow::queryClose()
 
     // Check what processes are running, excluding the shell
     QStringList processesRunning;
-    foreach (Session *session, _viewManager->sessions()) {
+    const auto uniqueSessions = QSet<Session*>::fromList(_viewManager->sessions());
+
+    foreach (Session *session, uniqueSessions) {
         if ((session == nullptr) || !session->isForegroundProcessActive()) {
             continue;
         }
@@ -642,7 +635,7 @@ bool MainWindow::queryClose()
     case KMessageBox::Yes:
         return true;
     case KMessageBox::No:
-        if ((_pluggedController != nullptr) && (_pluggedController->session() != nullptr)) {
+        if ((!_pluggedController.isNull()) && (!_pluggedController->session().isNull())) {
             disconnectController(_pluggedController);
             _pluggedController->session()->closeInNormalWay();
         }
@@ -771,17 +764,17 @@ void MainWindow::applyKonsoleSettings()
         removeMenuAccelerators();
     }
 
-    setNavigationVisibility(KonsoleSettings::tabBarVisibility());
-    setNavigationPosition(KonsoleSettings::tabBarPosition());
-    setNavigationBehavior(KonsoleSettings::newTabBehavior());
-    setNavigationTabWidthExpanding(KonsoleSettings::expandTabWidth());
-    setShowQuickButtons(KonsoleSettings::showQuickButtons());
+    _viewManager->setNavigationVisibility(KonsoleSettings::tabBarVisibility());
+    _viewManager->setNavigationPosition(KonsoleSettings::tabBarPosition());
+    _viewManager->setNavigationBehavior(KonsoleSettings::newTabBehavior());
+    _viewManager->setNavigationTabWidthExpanding(KonsoleSettings::expandTabWidth());
+    _viewManager->setShowQuickButtons(KonsoleSettings::showQuickButtons());
 
     if (KonsoleSettings::tabBarUseUserStyleSheet()) {
         setNavigationStyleSheetFromFile(KonsoleSettings::tabBarUserStyleSheetFile());
     } else {
         // Apply default values
-        setNavigationStyleSheet(KonsoleSettings::tabBarStyleSheet());
+        _viewManager->setNavigationStyleSheet(KonsoleSettings::tabBarStyleSheet());
     }
 
     setAutoSaveSettings(QStringLiteral("MainWindow"), KonsoleSettings::saveGeometryOnExit());
@@ -789,41 +782,17 @@ void MainWindow::applyKonsoleSettings()
     updateWindowCaption();
 }
 
-void MainWindow::setNavigationVisibility(int visibility)
-{
-    _viewManager->setNavigationVisibility(visibility);
-}
-
-void MainWindow::setNavigationPosition(int position)
-{
-    _viewManager->setNavigationPosition(position);
-}
-
-void MainWindow::setNavigationStyleSheet(const QString &styleSheet)
-{
-    _viewManager->setNavigationStyleSheet(styleSheet);
-}
-
-void MainWindow::setNavigationBehavior(int behavior)
-{
-    _viewManager->setNavigationBehavior(behavior);
-}
-
-void MainWindow::setNavigationTabWidthExpanding(bool expand)
-{
-    _viewManager->setNavigationTabWidthExpanding(expand);
-}
 
 void MainWindow::setNavigationStyleSheetFromFile(const QUrl &styleSheetFile)
 {
     // Let's only deal w/ local files for now
     if (!styleSheetFile.isLocalFile()) {
-        setNavigationStyleSheet(KonsoleSettings::tabBarStyleSheet());
+        _viewManager->setNavigationStyleSheet(KonsoleSettings::tabBarStyleSheet());
     }
 
     QFile file(styleSheetFile.toLocalFile());
     if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
-        setNavigationStyleSheet(KonsoleSettings::tabBarStyleSheet());
+        _viewManager->setNavigationStyleSheet(KonsoleSettings::tabBarStyleSheet());
     }
 
     QString styleSheetText;
@@ -833,12 +802,7 @@ void MainWindow::setNavigationStyleSheetFromFile(const QUrl &styleSheetFile)
     }
 
     // Replace current style sheet w/ loaded file
-    setNavigationStyleSheet(styleSheetText);
-}
-
-void MainWindow::setShowQuickButtons(bool show)
-{
-    _viewManager->setShowQuickButtons(show);
+    _viewManager->setNavigationStyleSheet(styleSheetText);
 }
 
 void MainWindow::activateMenuBar()
@@ -883,7 +847,7 @@ void MainWindow::configureNotifications()
 
 void MainWindow::setBlur(bool blur)
 {
-    if (_pluggedController == nullptr) {
+    if (_pluggedController.isNull()) {
         return;
     }
 
