@@ -99,6 +99,8 @@ SessionController::SessionController(Session* session , TerminalDisplay* view, Q
     , _view(view)
     , _copyToGroup(nullptr)
     , _profileList(nullptr)
+    , _sessionIcon(QIcon())
+    , _sessionIconName(QString())
     , _previousState(-1)
     , _searchFilter(nullptr)
     , _urlFilter(nullptr)
@@ -107,17 +109,21 @@ SessionController::SessionController(Session* session , TerminalDisplay* view, Q
     , _findAction(nullptr)
     , _findNextAction(nullptr)
     , _findPreviousAction(nullptr)
+    , _interactionTimer(nullptr)
     , _searchStartLine(0)
     , _prevSearchResultLine(0)
-    , _searchBar(nullptr)
     , _codecAction(nullptr)
     , _switchProfileMenu(nullptr)
     , _webSearchMenu(nullptr)
     , _listenForScreenWindowUpdates(false)
     , _preventClose(false)
     , _keepIconUntilInteraction(false)
+    , _selectedText(QString())
     , _showMenuAction(nullptr)
+    , _bookmarkValidProgramsToClear(QStringList())
     , _isSearchBarEnabled(false)
+    , _editProfileDialog(nullptr)
+    , _searchBar(view->searchBar())
 {
     Q_ASSERT(session);
     Q_ASSERT(view);
@@ -139,7 +145,7 @@ SessionController::SessionController(Session* session , TerminalDisplay* view, Q
     }
 
     setIdentifier(++_lastControllerId);
-    sessionTitleChanged();
+    sessionAttributeChanged();
 
     view->installEventFilter(this);
     view->setSessionController(this);
@@ -162,7 +168,7 @@ SessionController::SessionController(Session* session , TerminalDisplay* view, Q
     // listen to activity / silence notifications from session
     connect(_session.data(), &Konsole::Session::stateChanged, this, &Konsole::SessionController::sessionStateChanged);
     // listen to title and icon changes
-    connect(_session.data(), &Konsole::Session::titleChanged, this, &Konsole::SessionController::sessionTitleChanged);
+    connect(_session.data(), &Konsole::Session::sessionAttributeChanged, this, &Konsole::SessionController::sessionAttributeChanged);
     connect(_session.data(), &Konsole::Session::readOnlyChanged, this, &Konsole::SessionController::sessionReadOnlyChanged);
 
     connect(this, &Konsole::SessionController::tabRenamedByUser,  _session,  &Konsole::Session::tabRenamedByUser);
@@ -209,19 +215,19 @@ SessionController::SessionController(Session* session , TerminalDisplay* view, Q
     connect(_session.data(), &Konsole::Session::getBackgroundColor,
             this, &Konsole::SessionController::sendBackgroundColor);
 
-    connect(_session, &Konsole::Session::primaryScreenInUse, view, &Konsole::TerminalDisplay::usingPrimaryScreen);
-
     _allControllers.insert(this);
 
     // A list of programs that accept Ctrl+C to clear command line used
     // before outputting bookmark.
     _bookmarkValidProgramsToClear << QStringLiteral("bash") << QStringLiteral("fish") << QStringLiteral("sh");
     _bookmarkValidProgramsToClear << QStringLiteral("tcsh") << QStringLiteral("zsh");
+    setupSearchBar();
+    _searchBar->setVisible(_isSearchBarEnabled);
 }
 
 SessionController::~SessionController()
 {
-    if (_view != nullptr) {
+    if (!_view.isNull()) {
         _view->setScreenWindow(nullptr);
     }
 
@@ -255,7 +261,7 @@ void SessionController::interactionHandler()
 
 void SessionController::snapshot()
 {
-    Q_ASSERT(_session != nullptr);
+    Q_ASSERT(!_session.isNull());
 
     QString title = _session->getDynamicTitle();
     title         = title.simplified();
@@ -504,34 +510,16 @@ void SessionController::removeSearchFilter()
     _searchFilter = nullptr;
 }
 
-void SessionController::setSearchBar(IncrementalSearchBar* searchBar)
+void SessionController::setupSearchBar()
 {
-    // disconnect the existing search bar
-    if (_searchBar != nullptr) {
-        disconnect(this, nullptr, _searchBar, nullptr);
-        disconnect(_searchBar, nullptr, this, nullptr);
-    }
-
-    // connect new search bar
-    _searchBar = searchBar;
-    if (_searchBar != nullptr) {
-        connect(_searchBar.data(), &Konsole::IncrementalSearchBar::unhandledMovementKeyPressed, this, &Konsole::SessionController::movementKeyFromSearchBarReceived);
-        connect(_searchBar.data(), &Konsole::IncrementalSearchBar::closeClicked, this, &Konsole::SessionController::searchClosed);
-        connect(_searchBar.data(), &Konsole::IncrementalSearchBar::searchFromClicked, this, &Konsole::SessionController::searchFrom);
-        connect(_searchBar.data(), &Konsole::IncrementalSearchBar::findNextClicked, this, &Konsole::SessionController::findNextInHistory);
-        connect(_searchBar.data(), &Konsole::IncrementalSearchBar::findPreviousClicked, this, &Konsole::SessionController::findPreviousInHistory);
-        connect(_searchBar.data(), &Konsole::IncrementalSearchBar::highlightMatchesToggled , this , &Konsole::SessionController::highlightMatches);
-        connect(_searchBar.data(), &Konsole::IncrementalSearchBar::matchCaseToggled, this, &Konsole::SessionController::changeSearchMatch);
-        connect(_searchBar.data(), &Konsole::IncrementalSearchBar::matchRegExpToggled, this, &Konsole::SessionController::changeSearchMatch);
-
-        // if the search bar was previously active
-        // then re-enter search mode
-        enableSearchBar(_isSearchBarEnabled);
-    }
-}
-IncrementalSearchBar* SessionController::searchBar() const
-{
-    return _searchBar;
+    connect(_searchBar.data(), &Konsole::IncrementalSearchBar::unhandledMovementKeyPressed, this, &Konsole::SessionController::movementKeyFromSearchBarReceived);
+    connect(_searchBar.data(), &Konsole::IncrementalSearchBar::closeClicked, this, &Konsole::SessionController::searchClosed);
+    connect(_searchBar.data(), &Konsole::IncrementalSearchBar::searchFromClicked, this, &Konsole::SessionController::searchFrom);
+    connect(_searchBar.data(), &Konsole::IncrementalSearchBar::findNextClicked, this, &Konsole::SessionController::findNextInHistory);
+    connect(_searchBar.data(), &Konsole::IncrementalSearchBar::findPreviousClicked, this, &Konsole::SessionController::findPreviousInHistory);
+    connect(_searchBar.data(), &Konsole::IncrementalSearchBar::highlightMatchesToggled , this , &Konsole::SessionController::highlightMatches);
+    connect(_searchBar.data(), &Konsole::IncrementalSearchBar::matchCaseToggled, this, &Konsole::SessionController::changeSearchMatch);
+    connect(_searchBar.data(), &Konsole::IncrementalSearchBar::matchRegExpToggled, this, &Konsole::SessionController::changeSearchMatch);
 }
 
 void SessionController::setShowMenuAction(QAction* action)
@@ -858,7 +846,7 @@ void SessionController::renameSession()
 
     QPointer<Session> guard(_session);
     int result = dialog->exec();
-    if (guard == nullptr) {
+    if (guard.isNull()) {
         return;
     }
 
@@ -1081,7 +1069,7 @@ void SessionController::copyInputToSelectedTabs()
 
     QPointer<Session> guard(_session);
     int result = dialog->exec();
-    if (guard == nullptr) {
+    if (guard.isNull()) {
         return;
     }
 
@@ -1184,7 +1172,7 @@ void SessionController::listenForScreenWindowUpdates()
 
 void SessionController::updateSearchFilter()
 {
-    if ((_searchFilter != nullptr) && (_searchBar != nullptr)) {
+    if ((_searchFilter != nullptr) && (!_searchBar.isNull())) {
         _view->processFilters();
     }
 }
@@ -1206,7 +1194,7 @@ void SessionController::searchBarEvent()
 
 void SessionController::enableSearchBar(bool showSearchBar)
 {
-    if (_searchBar == nullptr) {
+    if (_searchBar.isNull()) {
         return;
     }
 
@@ -1226,7 +1214,7 @@ void SessionController::enableSearchBar(bool showSearchBar)
                    &Konsole::SessionController::findPreviousInHistory);
         disconnect(_searchBar.data(), &Konsole::IncrementalSearchBar::searchShiftPlusReturnPressed, this,
                    &Konsole::SessionController::findNextInHistory);
-        if ((_view != nullptr) && (_view->screenWindow() != nullptr)) {
+        if ((!_view.isNull()) && (_view->screenWindow() != nullptr)) {
             _view->screenWindow()->setCurrentResultLine(-1);
         }
     }
@@ -1267,7 +1255,7 @@ void SessionController::searchHistory(bool showSearchBar)
 {
     enableSearchBar(showSearchBar);
 
-    if (_searchBar != nullptr) {
+    if (!_searchBar.isNull()) {
         if (showSearchBar) {
             removeSearchFilter();
 
@@ -1317,7 +1305,7 @@ void SessionController::searchCompleted(bool success)
 {
     _prevSearchResultLine = _view->screenWindow()->currentResultLine();
 
-    if (_searchBar != nullptr) {
+    if (!_searchBar.isNull()) {
         _searchBar->setFoundMatch(success);
     }
 }
@@ -1330,7 +1318,7 @@ void SessionController::beginSearch(const QString& text, Enum::SearchDirection d
     QRegularExpression regExp = regexpFromSearchBarOptions();
     _searchFilter->setRegExp(regExp);
 
-    if (_searchStartLine == -1) {
+    if (_searchStartLine < 0 || _searchStartLine > _view->screenWindow()->lineCount()) {
         if (direction == Enum::ForwardsSearch) {
             setSearchStartTo(_view->screenWindow()->currentLine());
         } else {
@@ -1427,7 +1415,7 @@ void SessionController::showHistoryOptions()
 
     QPointer<Session> guard(_session);
     int result = dialog->exec();
-    if (guard == nullptr) {
+    if (guard.isNull()) {
         return;
     }
 
@@ -1495,6 +1483,7 @@ void SessionController::clearHistory()
 {
     _session->clearHistory();
     _view->updateImage();   // To reset view scrollbar
+    _view->repaint();
 }
 
 void SessionController::clearHistoryAndReset()
@@ -1576,14 +1565,14 @@ void SessionController::updateReadOnlyActionStates()
 
 bool SessionController::isReadOnly() const
 {
-    if (_session != nullptr) {
+    if (!_session.isNull()) {
         return _session->isReadOnly();
     } else {
         return false;
     }
 }
 
-void SessionController::sessionTitleChanged()
+void SessionController::sessionAttributeChanged()
 {
     if (_sessionIconName != _session->iconName()) {
         _sessionIconName = _session->iconName();
@@ -1610,7 +1599,7 @@ void SessionController::sessionTitleChanged()
 
 void SessionController::sessionReadOnlyChanged() {
     // Trigger icon update
-    sessionTitleChanged();
+    sessionAttributeChanged();
 
     updateReadOnlyActionStates();
 
@@ -1637,7 +1626,7 @@ void SessionController::showDisplayContextMenu(const QPoint& position)
     }
 
     QPointer<QMenu> popup = qobject_cast<QMenu*>(factory()->container(QStringLiteral("session-popup-menu"), this));
-    if (popup != nullptr) {
+    if (!popup.isNull()) {
         updateReadOnlyActionStates();
 
         // prepend content-specific actions such as "Open Link", "Copy Email Address" etc.
@@ -1665,7 +1654,7 @@ void SessionController::showDisplayContextMenu(const QPoint& position)
         QAction* chosen = popup->exec(_view->mapToGlobal(position));
 
         // check for validity of the pointer to the popup menu
-        if (popup != nullptr) {
+        if (!popup.isNull()) {
             // Remove content-specific actions
             //
             // If the close action was chosen, the popup menu will be partially
@@ -1892,7 +1881,7 @@ void SaveHistoryTask::jobDataRequested(KIO::Job* job , QByteArray& data)
 
     // transfer LINES_PER_REQUEST lines from the session's history
     // to the save location
-    if (info.session != nullptr) {
+    if (!info.session.isNull()) {
         // note:  when retrieving lines from the emulation,
         // the first line is at index 0.
 
@@ -2120,7 +2109,7 @@ QRegularExpression SearchHistoryTask::regExp() const
 
 QString SessionController::userTitle() const
 {
-    if (_session != nullptr) {
+    if (!_session.isNull()) {
         return _session->userTitle();
     } else {
         return QString();
