@@ -46,13 +46,13 @@ class QScrollBar;
 class QShowEvent;
 class QHideEvent;
 class QTimerEvent;
-
 class KMessageWidget;
 
 namespace Konsole {
 class FilterChain;
 class TerminalImageFilterChain;
 class SessionController;
+class IncrementalSearchBar;
 /**
  * A widget which displays output from a terminal emulation and sends input keypresses and mouse activity
  * to the terminal.
@@ -263,6 +263,18 @@ public:
      */
     Enum::CursorShapeEnum keyboardCursorShape() const;
 
+    /**
+     * Sets the Cursor Style (DECSCUSR) via escape sequences
+     * @p shape cursor shape
+     * @p isBlinking if true, the cursor will be set to blink
+    */
+    void setCursorStyle(Enum::CursorShapeEnum shape, bool isBlinking);
+    /**
+     * Resets the cursor style to the current profile cursor shape and
+     * blinking settings
+    */
+    void resetCursorStyle();
+
     /**
      * Sets the color used to draw the keyboard cursor.
      *
@@ -552,8 +564,11 @@ public:
         return _flowControlWarningEnabled;
     }
 
-    /** See setUsesMouse() */
-    bool usesMouse() const;
+    /** See setUsesMouseTracking() */
+    bool usesMouseTracking() const;
+
+    /** See setAlternateScrolling() */
+    bool alternateScrolling() const;
 
 public Q_SLOTS:
     /**
@@ -613,29 +628,30 @@ public Q_SLOTS:
     void outputSuspended(bool suspended);
 
     /**
-     * Sets whether the program whose output is being displayed in the view
-     * is interested in mouse events.
+     * Sets whether the program currently running in the terminal is interested
+     * in Mouse Tracking events.
      *
-     * If this is set to true, mouse signals will be emitted by the view when the user clicks, drags
-     * or otherwise moves the mouse inside the view.
-     * The user interaction needed to create selections will also change, and the user will be required
-     * to hold down the shift key to create a selection or perform other mouse activities inside the
-     * view area - since the program running in the terminal is being allowed to handle normal mouse
-     * events itself.
+     * When set to true, Konsole will send Mouse Tracking events.
      *
-     * @param on Set to true if the program running in the terminal is interested in mouse events
-     * or false otherwise.
+     * The user interaction needed to create text selections will change
+     * also, and the user will be required to hold down the Shift key to
+     * create a selection or perform other mouse activities inside the view
+     * area, since the program running in the terminal is being allowed
+     * to handle normal mouse events itself.
+     *
+     * @param on Set to true if the program running in the terminal is
+     * interested in Mouse Tracking events or false otherwise.
      */
-    void setUsesMouse(bool on);
+    void setUsesMouseTracking(bool on);
 
     /**
-     * Sets _isPrimaryScreen depending on which screen is currently in
-     * use, primary or alternate
-     *
-     * @param use Set to @c true if the primary screen is in use or to
-     * @c false otherwise (i.e. the alternate screen is in use)
+     * Sets the AlternateScrolling profile property which controls whether
+     * to emulate up/down key presses for mouse scroll wheel events.
+     * For more details, check the documentation of that property in the
+     * Profile header.
+     * Enabled by default.
      */
-    void usingPrimaryScreen(bool use);
+    void setAlternateScrolling(bool enable);
 
     void setBracketedPasteMode(bool on);
 
@@ -669,7 +685,7 @@ public Q_SLOTS:
 
     // Used to show/hide the message widget
     void updateReadOnlyState(bool readonly);
-
+    IncrementalSearchBar *searchBar() const;
 Q_SIGNALS:
 
     /**
@@ -689,8 +705,8 @@ Q_SIGNALS:
     void changedContentSizeSignal(int height, int width);
 
     /**
-     * Emitted when the user right clicks on the display, or right-clicks with the Shift
-     * key held down if usesMouse() is true.
+     * Emitted when the user right clicks on the display, or right-clicks
+     * with the Shift key held down if usesMouseTracking() is true.
      *
      * This can be used to display a context menu.
      */
@@ -770,7 +786,6 @@ protected:
     QVariant inputMethodQuery(Qt::InputMethodQuery query) const Q_DECL_OVERRIDE;
 
     void updateScrollBarPalette();
-
 protected Q_SLOTS:
 
     void scrollBarPositionChanged(int value);
@@ -835,8 +850,9 @@ private:
     QRect imageToWidget(const QRect &imageArea) const;
 
     // maps a point on the widget to the position ( ie. line and column )
-    // of the character at that point.
-    void getCharacterPosition(const QPoint &widgetPoint, int &line, int &column) const;
+    // of the character at that point. When the edge is true, it maps to
+    // a character which left edge is closest to the point.
+    void getCharacterPosition(const QPoint &widgetPoint, int &line, int &column, bool edge) const;
 
     // the area where the preedit string for input methods will be draw
     QRect preeditRect() const;
@@ -867,6 +883,9 @@ private:
     // returns the position of the cursor in columns and lines
     QPoint cursorPosition() const;
 
+    // returns true if the cursor's position is on display.
+    bool isCursorOnDisplay() const;
+
     // redraws the cursor
     void updateCursor();
 
@@ -887,6 +906,8 @@ private:
     // Boilerplate setup for MessageWidget
     KMessageWidget* createMessageWidget(const QString &text);
 
+    int loc(int x, int y) const;
+
     // the window onto the terminal screen which this display
     // is currently showing.
     QPointer<ScreenWindow> _screenWindow;
@@ -901,9 +922,6 @@ private:
     int _fontAscent;      // ascend
     bool _boldIntense;   // Whether intense colors should be rendered with bold font
 
-    int _leftMargin;    // offset
-    int _topMargin;    // offset
-
     int _lines;      // the number of lines that can be displayed in the widget
     int _columns;    // the number of columns that can be displayed in the widget
 
@@ -928,8 +946,8 @@ private:
     bool _resizing;
     bool _showTerminalSizeHint;
     bool _bidiEnabled;
-    bool _mouseMarks;
-    bool _isPrimaryScreen;
+    bool _usesMouseTracking;
+    bool _alternateScrolling;
     bool _bracketedPasteMode;
 
     QPoint _iPntSel;  // initial selection point
@@ -1033,6 +1051,7 @@ private:
     qreal _opacity;
 
     ScrollState _scrollWheelState;
+    IncrementalSearchBar *_searchBar;
 
     friend class TerminalDisplayAccessible;
 };
