@@ -43,7 +43,10 @@ ViewContainerTabBar::ViewContainerTabBar(QWidget *parent, TabbedViewContainer *c
     _dropIndicator(nullptr),
     _dropIndicatorIndex(-1),
     _drawIndicatorDisabled(false),
-    _connectedContainer(container)
+    _supportedMimeType(QString()),
+    _connectedContainer(container),
+    _dragStart(QPoint()),
+    _mousePressTimer(nullptr)
 {
     setDrawBase(true);
     setDocumentMode(true);
@@ -262,7 +265,7 @@ QPixmap ViewContainerTabBar::dragDropPixmap(int tab)
     QPixmap tabPixmap(rect.width() + borderWidth,
                       rect.height() + borderWidth);
     QPainter painter(&tabPixmap);
-    painter.drawPixmap(0, 0, this->grab(rect));
+    painter.drawPixmap(0, 0, grab(rect));
     QPen borderPen;
     borderPen.setBrush(palette().dark());
     borderPen.setWidth(borderWidth);
